@@ -120,6 +120,7 @@ if(FALSE)##--- no longer used:
     ## block-diagonal matrix [a dgTMatrix] from list of matrices
     stopifnot(is.list(lst), (nl <- length(lst)) >= 1)
 
+### FIXME: next line is *slow* when lst = list of 75'000  dense 3x3 matrices
     Tlst <- lapply(lapply(lst, as_Csp2), # includes "diagU2N"
 		   as, "TsparseMatrix")
     if(nl == 1) return(Tlst[[1]])
@@ -350,7 +351,7 @@ setAs("matrix", "diagonalMatrix",
 	  if(d[1] != (n <- d[2])) stop("non-square matrix")
 	  if(any(from[row(from) != col(from)] != 0))
 	      stop("matrix with non-zero off-diagonals cannot be coerced to \"diagonalMatrix\"")
-	  x <- diag(from)
+	  x <- diag(from); names(x) <- NULL # don't want them in 'x' slot
 	  if(is.logical(x)) {
 	      cl <- "ldiMatrix"
 	      uni <- allTrue(x) ## uni := {is it unit-diagonal ?}
@@ -370,7 +371,7 @@ setAs("Matrix", "diagonalMatrix",
 	  if(d[1] != (n <- d[2])) stop("non-square matrix")
 	  if(!isDiagonal(from)) stop("matrix is not diagonal")
 	  ## else:
-	  x <- diag(from)
+	  x <- diag(from); names(x) <- NULL # don't want them in 'x' slot
 	  if(is.logical(x)) {
 	      cl <- "ldiMatrix"
 	      uni <- allTrue(x)
@@ -555,7 +556,7 @@ setMethod("norm", signature(x = "diagonalMatrix", type = "character"),
 	      type <- toupper(substr(type[1], 1, 1))
 	      isU <- (x@diag == "U") # unit-diagonal
 	      if(type == "F") sqrt(if(isU) n else sum(x@x^2))
-	      else { ## norm == "I","1","O","M" :
+	      else { ## norm == "I","1","O","M","2" :
 		  if(isU) 1 else max(abs(x@x))
 	      }
 	  })
