@@ -662,10 +662,11 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
     enum dense_enum M_type = ddense /* -Wall */;
 
     if (ctype > 0) { /* a [nld]denseMatrix or [dl]diMatrix object */
-	ad = GET_SLOT(A, Matrix_DimSym);
-	an = GET_SLOT(A, Matrix_DimNamesSym);
 	M_type = (ctype <= 14) ? ddense :
 	    ((ctype <= 14+6) ? ldense : ndense);
+	/// TODO: Return 'A' unduplicated if ctype indicates "?geMatrix"' -- and *new argument* 'check' is TRUE
+	ad = GET_SLOT(A, Matrix_DimSym);
+	an = GET_SLOT(A, Matrix_DimNamesSym);
     }
     else if (ctype < 0) {	/* not a (recognized) classed matrix */
 
@@ -712,10 +713,10 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
     ans = PROTECT(NEW_OBJECT(MAKE_CLASS(M_type == ddense ? "dgeMatrix" :
 					(M_type == ldense ? "lgeMatrix" :
 					 "ngeMatrix"))));
-#define DUP_MMATRIX_SET_1					\
-    SET_SLOT(ans, Matrix_DimSym, duplicate(ad));		\
-    SET_SLOT(ans, Matrix_DimNamesSym, (LENGTH(an) == 2) ? 	\
-	     duplicate(an): allocVector(VECSXP, 2));		\
+#define DUP_MMATRIX_SET_1						\
+    SET_SLOT(ans, Matrix_DimSym, duplicate(ad));			\
+    SET_SLOT(ans, Matrix_DimNamesSym, (!isNull(an) && LENGTH(an) == 2) ? \
+	     duplicate(an): allocVector(VECSXP, 2));			\
     sz = INTEGER(ad)[0] * INTEGER(ad)[1]
 
     DUP_MMATRIX_SET_1;
