@@ -1,5 +1,7 @@
 #  File src/library/base/R/conditions.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
+#
+#  Copyright (C) 1995-2012 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -12,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 ##
 ## Handling Conditions
@@ -123,18 +125,18 @@ as.character.condition <- function(x, ...) {
     call <- conditionCall(x)
     cl <- class(x)[1L]
     if (! is.null(call))
-        paste(cl, " in ", deparse(call)[1L], ": ", msg, "\n", sep="")
+        paste0(cl, " in ", deparse(call)[1L], ": ", msg, "\n")
     else
-        paste(cl, ": ", msg, "\n", sep="")
+        paste0(cl, ": ", msg, "\n")
 }
 
 as.character.error <- function(x, ...) {
     msg <- conditionMessage(x)
     call <- conditionCall(x)
     if (! is.null(call))
-        paste("Error in ", deparse(call)[1L], ": ", msg, "\n", sep="")
+        paste0("Error in ", deparse(call)[1L], ": ", msg, "\n")
     else
-        paste("Error: ", msg, "\n", sep="")
+        paste0("Error: ", msg, "\n")
 }
 
 signalCondition <- function(cond) {
@@ -218,7 +220,7 @@ invokeRestartInteractively <- function(r) {
 		    args <- c(args, eval(parse(prompt = prompt)))
 		}
 		else {
-		    prompt <- paste(p, ": ", sep="")
+		    prompt <- paste0(p, ": ")
 		    args <- c(args, list(eval(parse(prompt = prompt))))
 		}
 	    }
@@ -307,3 +309,9 @@ withRestarts <- function(expr, ...) {
 
 .handleSimpleError <- function(h, msg, call)
     h(simpleError(msg, call))
+
+.tryResumeInterrupt <- function() {
+    r <- findRestart("resume")
+    if (! is.null(r))
+        invokeRestart(r)
+}
