@@ -1,5 +1,5 @@
 #  File src/library/graphics/R/image.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
 #  Copyright (C) 1995-2014 The R Core Team
 #
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 image <- function(x, ...) UseMethod("image")
 
@@ -44,8 +44,8 @@ image.default <- function (x = seq(0, 1, length.out = nrow(z)),
 	} else stop("no 'z' matrix specified")
     } else if (is.list(x)) {
 	xn <- deparse(substitute(x))
-	if (missing(xlab)) xlab <- paste(xn, "x", sep = "$")
-	if (missing(ylab)) ylab <- paste(xn, "y", sep = "$")
+	if (missing(xlab)) xlab <- paste0(xn, "$x")
+	if (missing(ylab)) ylab <- paste0(xn, "$y")
 	y <- x$y
 	x <- x$x
     } else {
