@@ -1,5 +1,5 @@
 #  File src/library/parallel/R/unix/mcfork.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
 #  Copyright (C) 1995-2013 The R Core Team
 #
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 ### Derived from multicore version 0.1-6 by Simon Urbanek
 
@@ -28,7 +28,12 @@ clean_pids <- function(e)
 
 mcfork <- function(estranged = FALSE) {
     r <- .Call(C_mc_fork, estranged)
-    processClass <- if (!r[1L]) "masterProcess" else
+
+    # Disable JIT in the child process because it could lead to repeated
+    # compilation of the same functions in each forked R process. Ideally
+    # the compiled code would propagate to other processes, but it is not
+    # currently possible.
+    processClass <- if (!r[1L]) { compiler::enableJIT(0) ; "masterProcess" }  else
     		    if (is.na(r[2L])) "estrangedProcess" else "childProcess"
     structure(list(pid = r[1L], fd = r[2:3]), class = c(processClass, "process"))
 }
@@ -89,7 +94,7 @@ processID <- function(process) {
               domain = NA)
 }
 
-# unused
+# unused in the package
 sendChildStdin <- function(child, what)
 {
     if (inherits(child, "process") || is.list(child)) child <- processID(child)
