@@ -1,5 +1,5 @@
 #  File src/library/parallel/R/unix/mclapply.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
 #  Copyright (C) 1995-2014 The R Core Team
 #
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 ### Derived from multicore version 0.1-6 by Simon Urbanek
 
@@ -22,7 +22,6 @@ mclapply <- function(X, FUN, ..., mc.preschedule = TRUE, mc.set.seed = TRUE,
                      mc.silent = FALSE, mc.cores = getOption("mc.cores", 2L),
                      mc.cleanup = TRUE, mc.allow.recursive = TRUE)
 {
-    env <- parent.frame()
     cores <- as.integer(mc.cores)
     if(is.na(cores) || cores < 1L) stop("'mc.cores' must be >= 1")
     .check_ncores(cores)
@@ -31,6 +30,7 @@ mclapply <- function(X, FUN, ..., mc.preschedule = TRUE, mc.set.seed = TRUE,
         return(lapply(X = X, FUN = FUN, ...))
 
     if(mc.set.seed) mc.reset.stream()
+    if(!length(X)) return(list())
 
     jobs <- list()
     cleanup <- function() {
