@@ -1,7 +1,7 @@
 #  File src/library/tools/R/Rd2txt.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2016 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 ## This stops on
 ##  unrecognized tag
@@ -27,7 +27,7 @@ tabExpand <- function(x) {
     srcref <- attr(x, "srcref")
     if (is.null(srcref)) start <- 0L
     else start <- srcref[5L] - 1L
-    .Call(doTabExpand, x, start)
+    .Call(C_doTabExpand, x, start)
 }
 
 Rd2txt_options <- local({
@@ -396,9 +396,6 @@ Rd2txt <-
         linestart <<- trail
     }
 
-    blanks <- function(n)
-	if (n) paste(rep_len(" ", n), collapse="") else ""
-
     flushBuffer <- function() {
     	if (!length(buffer)) return()
 
@@ -434,10 +431,10 @@ Rd2txt <-
 	} else {  # Not wrapping
 	    if (keepFirstIndent) {
 		if (length(buffer) > 1L)
-		    buffer[-1L] <<- paste0(blanks(indent), buffer[-1L])
+		    buffer[-1L] <<- paste0(strrep(" ", indent), buffer[-1L])
 		keepFirstIndent <- FALSE
 	    } else
-		buffer <<- paste0(blanks(indent), buffer)
+		buffer <<- paste0(strrep(" ", indent), buffer)
 	}
 
     	if (length(buffer)) WriteLines(buffer, con, outputEncoding)
@@ -774,7 +771,7 @@ Rd2txt <-
         }
         result <- matrix("", sum(lines), cols)
         for (i in seq_len(cols))
-            result[, i] <- blanks(widths[i])
+            result[, i] <- strrep(" ", widths[i])
         firstline <- c(1L, 1L+cumsum(lines))
         for (i in seq_along(entries)) {
             e <- entries[[i]]
@@ -859,7 +856,7 @@ Rd2txt <-
                                   indent <<- max(opts$minIndent,
                                                  indent + opts$extraIndent)
                                   keepFirstIndent <<- TRUE
-                                  putw(paste(rep_len(" ", indent0), collapse=""),
+                                  putw(strrep(" ", indent0),
                                        frmt(paste0(DLlab),
                                             justify="left", width=indent),
                                        " ")
@@ -918,7 +915,7 @@ Rd2txt <-
             return()
     	save <- c(indent, sectionLevel, keepFirstIndent, dropBlank, wrapping)
     	blankLine(min(sectionLevel, 1L))
-    	titlePrefix <- paste(rep("  ", sectionLevel), collapse="")
+    	titlePrefix <- strrep("  ", sectionLevel)
     	opts <- Rd2txt_options()
         indent <<- opts$sectionIndent + opts$sectionExtra*sectionLevel
         sectionLevel <<- sectionLevel + 1
@@ -991,8 +988,8 @@ Rd2txt <-
 		right <- paste0(right, "(", encoding, ")")
 	    pad <- max(HDR_WIDTH - nchar(left, "w") - nchar(mid, "w") - nchar(right, "w"), 0)
 	    pad0 <- pad %/% 2L
-	    pad1 <- paste(rep.int(" ", pad0), collapse = "")
-	    pad2 <- paste(rep.int(" ", pad - pad0), collapse = "")
+	    pad1 <- strrep(" ", pad0)
+	    pad2 <- strrep(" ", pad - pad0)
 	    putf(paste0(left, pad1, mid, pad2, right, "\n\n"))
 	}
 
