@@ -1,7 +1,7 @@
 #  File src/library/tools/R/urltools.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 2015 The R Core Team
+#  Copyright (C) 2015-2017 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -19,25 +19,16 @@
 get_IANA_URI_scheme_db <-
 function()
 {
-    ## See <http://www.iana.org/assignments/uri-schemes/uri-schemes.xhtml>.
+    ## See
+    ## <http://www.iana.org/assignments/uri-schemes/uri-schemes.xhtml>.
     baseurl <- "http://www.iana.org/assignments/uri-schemes/"
-    permanent <- utils::read.csv(url(paste0(baseurl, "uri-schemes-1.csv")),
-                                 stringsAsFactors = FALSE)
-    provisional <- utils::read.csv(url(paste0(baseurl, "uri-schemes-2.csv")),
-                                   stringsAsFactors = FALSE)
-    historical <- utils::read.csv(url(paste0(baseurl, "uri-schemes-3.csv")),
-                                  stringsAsFactors = FALSE)
-    db <- rbind(permanent, provisional, historical)
-    db$Category <-
-        rep.int(c("permanent", "provisional", "historical"),
-                c(nrow(permanent),
-                  nrow(provisional),
-                  nrow(historical)))
+    db <- utils::read.csv(url(paste0(baseurl, "uri-schemes-1.csv")),
+                          stringsAsFactors = FALSE)
     names(db) <- chartr(".", "_", names(db))
     db
 }
 
-parse_URL_reference <-
+parse_URI_reference <-
 function(x)
 {
     ## See RFC_3986 <http://www.ietf.org/rfc/rfc3986.txt>.
@@ -78,23 +69,9 @@ function(x)
 .get_urls_from_HTML_file <-
 function(f)
 {
-    hrefs <- character()
-    XML::htmlParse(f,
-                   handlers =
-                       list(a = function(node) {
-                           href <- XML::xmlAttrs(node)["href"]
-                           ## <FIXME>
-                           ## ? XML::xmlAttrs says the value is always a
-                           ## named character vector, but
-                           ##   XML::xmlAttrs(XML::xmlNode("a", "foobar"))
-                           ## gives NULL ...
-                           ## Hence, use an extra is.null() test.
-                           if(!is.null(href) && !is.na(href))
-                               hrefs <<- c(hrefs, href)
-                           ## </FIXME>
-                       })
-                   )
-    unique(unname(hrefs[!grepl("^#", hrefs)]))
+    nodes <- xml2::xml_find_all(xml2::read_html(f), "//a")
+    hrefs <- xml2::xml_attr(nodes, "href")
+    unique(hrefs[!is.na(hrefs) & !startsWith(hrefs, "#")])
 }
 
 url_db <-
@@ -120,16 +97,6 @@ function(db)
 url_db_from_package_metadata <-
 function(meta)
 {
-    gregexec_at_pos <- function(pattern, v, m, pos) {
-        unlist(lapply(regmatches(v, m),
-                      function(e)
-                          do.call(rbind,
-                                  regmatches(e,
-                                             regexec(pattern, e)))[, 3L]
-                      ),
-               use.names = FALSE)
-    }
-
     urls <- character()
     fields <- c("URL", "BugReports")
     for(v in meta[fields]) {
@@ -137,11 +104,11 @@ function(meta)
         pattern <-
             "<(URL: *)?((https?|ftp)://[^[:space:],]*)[[:space:]]>"
         m <- gregexpr(pattern, v)
-        urls <- c(urls, gregexec_at_pos(pattern, v, m, 3L))
+        urls <- c(urls, .gregexec_at_pos(pattern, v, m, 3L))
         regmatches(v, m) <- ""
         pattern <- "(^|[^>\"])((https?|ftp)://[^[:space:],]*)"
         m <- gregexpr(pattern, v)
-        urls <- c(urls, gregexec_at_pos(pattern, v, m, 3L))
+        urls <- c(urls, .gregexec_at_pos(pattern, v, m, 3L))
     }
 
     url_db(urls, rep.int("DESCRIPTION", length(urls)))
@@ -168,8 +135,7 @@ function(dir, installed = FALSE)
     path <- if(installed) "NEWS.Rd" else file.path("inst", "NEWS.Rd")
     nfile <- file.path(dir, path)
     if(file.exists(nfile)) {
-        macros <- loadRdMacros(file.path(R.home("share"),
-                                         "Rd", "macros", "system.Rd"))
+        macros <- initialRdMacros()
         urls <- .get_urls_from_Rd(prepare_Rd(tools::parse_Rd(nfile,
                                                              macros = macros),
                                              stages = "install"))
@@ -200,20 +166,43 @@ function(dir, installed = FALSE)
 }
 
 url_db_from_package_README_md <-
-function(dir)
+function(dir, installed = FALSE)
 {
-    urls <- character()
-    if(file.exists(rfile <- file.path(dir, "README.md")) &&
-       nzchar(Sys.which("pandoc"))) {
-        tfile <- tempfile("README", fileext=".html")
+    urls <- path <- character()
+    rfile <- Filter(file.exists,
+                    c(if(!installed)
+                          file.path(dir, "inst", "README.md"),
+                      file.path(dir, "README.md")))[1L]
+    if(!is.na(rfile) && nzchar(Sys.which("pandoc"))) {
+        path <- .file_path_relative_to_dir(rfile, dir)
+        tfile <- tempfile("README", fileext = ".html")
         on.exit(unlink(tfile))
-        out <- .pandoc_README_md_for_CRAN(rfile, tfile)
+        out <- .pandoc_md_for_CRAN(rfile, tfile)
         if(!out$status) {
             urls <- .get_urls_from_HTML_file(tfile)
         }
     }
-    url_db(urls, rep.int("README.md", length(urls)))
+    url_db(urls, rep.int(path, length(urls)))
+}
 
+url_db_from_package_NEWS_md <-
+function(dir, installed = FALSE)
+{
+    urls <- path <- character()
+    nfile <- Filter(file.exists,
+                    c(if(!installed)
+                          file.path(dir, "inst", "NEWS.md"),
+                      file.path(dir, "NEWS.md")))[1L]
+    if(!is.na(nfile) && nzchar(Sys.which("pandoc"))) {
+        path <- .file_path_relative_to_dir(nfile, dir)
+        tfile <- tempfile("NEWS", fileext = ".html")
+        on.exit(unlink(tfile))
+        out <- .pandoc_md_for_CRAN(nfile, tfile)
+        if(!out$status) {
+            urls <- .get_urls_from_HTML_file(tfile)
+        }
+    }
+    url_db(urls, rep.int(path, length(urls)))
 }
 
 url_db_from_package_sources <-
@@ -223,10 +212,12 @@ function(dir, add = FALSE) {
                 url_db_from_package_Rd_db(Rd_db(dir = dir)),
                 url_db_from_package_citation(dir, meta),
                 url_db_from_package_news(dir))
-    if(requireNamespace("XML", quietly = TRUE)) {
+    if(requireNamespace("xml2", quietly = TRUE)) {
         db <- rbind(db,
                     url_db_from_package_HTML_files(dir),
-                    url_db_from_package_README_md(dir))
+                    url_db_from_package_README_md(dir),
+                    url_db_from_package_NEWS_md(dir)
+                    )
     }
     if(add)
         db$Parent <- file.path(basename(dir), db$Parent)
@@ -249,10 +240,15 @@ function(packages, lib.loc = NULL, verbose = FALSE)
                     url_db_from_package_citation(dir, meta,
                                                  installed = TRUE),
                     url_db_from_package_news(dir, installed = TRUE))
-        if(requireNamespace("XML", quietly = TRUE)) {
+        if(requireNamespace("xml2", quietly = TRUE)) {
             db <- rbind(db,
                         url_db_from_package_HTML_files(dir,
-                                                       installed = TRUE))
+                                                       installed = TRUE),
+                        url_db_from_package_README_md(dir,
+                                                      installed = TRUE),
+                        url_db_from_package_NEWS_md(dir,
+                                                    installed = TRUE)
+                        )
         }
         db$Parent <- file.path(p, db$Parent)
         db
@@ -312,16 +308,23 @@ table_of_FTP_server_return_codes <-
       )
 
 check_url_db <-
-function(db, verbose = FALSE)
+function(db, remote = TRUE, verbose = FALSE)
 {
+    use_curl <-
+        config_val_to_logical(Sys.getenv("_R_CHECK_URLS_USE_CURL_",
+                                         "TRUE")) &&
+        requireNamespace("curl", quietly = TRUE)
+
     .gather <- function(u = character(),
                         p = list(),
                         s = rep.int("", length(u)),
                         m = rep.int("", length(u)),
+                        new = rep.int("", length(u)),
                         cran = rep.int("", length(u)),
-                        spaces = rep.int("", length(u))) {
+                        spaces = rep.int("", length(u)),
+                        R = rep.int("", length(u))) {
         y <- data.frame(URL = u, From = I(p), Status = s, Message = m,
-                        CRAN = cran, Spaces = spaces,
+                        New = new, CRAN = cran, Spaces = spaces, R = R,
                         stringsAsFactors = FALSE)
         y$From <- p
         class(y) <- c("check_url_db", "data.frame")
@@ -332,6 +335,9 @@ function(db, verbose = FALSE)
         if(verbose) message(sprintf("processing %s", u))
         h <- tryCatch(curlGetHeaders(u), error = identity)
         if(inherits(h, "error")) {
+            ## Currently, this info is only used in .check_http().
+            ## Might be useful for checking ftps too, so simply leave it
+            ## here instead of moving to .check_http().
             msg <- conditionMessage(h)
             if (grepl("libcurl error code (51|60)", msg)) {
                 h2 <- tryCatch(curlGetHeaders(u, verify = FALSE),
@@ -354,7 +360,12 @@ function(db, verbose = FALSE)
         c(s, msg, "", "")
     }
 
-    .check_http <- function(u) {
+    .check_http <- if(remote)
+        function(u) c(.check_http_A(u), .check_http_B(u))
+    else
+        function(u) c(rep.int("", 3L), .check_http_B(u))
+
+    .check_http_A <- function(u) {
         h <- .fetch(u)
         newLoc <- ""
         if(inherits(h, "error")) {
@@ -375,22 +386,56 @@ function(db, verbose = FALSE)
             if (length(ind))
                 newLoc <- sub("^[Ll]ocation: ([^\r]*)\r\n", "\\1", h[max(ind)])
         }
+        ##
+        if((s != "200") && use_curl) {
+            g <- .curl_GET_status(u)
+            if(g == "200") {
+                s <- g
+                msg <- "OK"
+            }
+        }
         ## A mis-configured site
         if (s == "503" && any(grepl("www.sciencedirect.com", c(u, newLoc))))
             s <- "405"
-        cran <- grepl("https?://cran.r-project.org/web/packages/[.[:alnum:]]+(|/|/index.html)$",
-                      u, ignore.case = TRUE)
+        c(s, msg, newLoc)
+    }
+
+    .check_http_B <- function(u) {
+        ul <- tolower(u)
+        cran <- ((grepl("^https?://cran.r-project.org/web/packages", ul) &&
+                  !grepl("^https?://cran.r-project.org/web/packages/[.[:alnum:]]+(html|pdf|rds)$",
+                         ul)) ||
+                 (grepl("^https?://cran.r-project.org/web/views/[[:alnum:]]+[.]html$",
+                        ul)) ||
+                 startsWith(ul, "http://cran.r-project.org") ||
+                 any(substring(ul, 1L, nchar(mirrors)) == mirrors))
+        R <- grepl("^http://(www|bugs|journal).r-project.org", ul)
         spaces <- grepl(" ", u)
-        c(s, msg, newLoc, if(cran) u else "", if(spaces) u else "")
+        c(if(cran) u else "", if(spaces) u else "", if(R) u else "")
     }
 
     bad <- .gather()
 
     if(!NROW(db)) return(bad)
 
-    parents <- split(db$Parent, db$URL)
-    urls <- names(parents)
-    parts <- parse_URL_reference(urls)
+    ## Could also use utils::getCRANmirrors(local.only = TRUE).
+    mirrors <- c(utils::read.csv(file.path(R.home("doc"),
+                                           "CRAN_mirrors.csv"),
+                                 as.is = TRUE, encoding = "UTF-8")$URL,
+                 "http://cran.rstudio.com/",
+                 "https://cran.rstudio.com/")
+    mirrors <- tolower(sub("/$", "", mirrors))
+
+    if(inherits(db, "check_url_db")) {
+        ## Allow re-checking check results.
+        parents <- db$From
+        urls <- db$URL
+    } else {
+        parents <- split(db$Parent, db$URL)
+        urls <- names(parents)
+    }
+
+    parts <- parse_URI_reference(urls)
 
     ## Empty URLs.
     ind <- apply(parts == "", 1L, all)
@@ -404,18 +449,26 @@ function(db, verbose = FALSE)
 
     ## Invalid URI schemes.
     schemes <- parts[, 1L]
-    ind <- is.na(match(schemes, c("", IANA_URI_scheme_db$URI_Scheme)))
+    ind <- is.na(match(schemes,
+                       c("",
+                         IANA_URI_scheme_db$URI_Scheme,
+                         ## Also allow 'javascript' scheme, see
+                         ## <https://tools.ietf.org/html/draft-hoehrmann-javascript-scheme-03>
+                         ## (but apparently never registered with IANA).
+                         "javascript")))
     if(any(ind)) {
         len <- sum(ind)
+        msg <- rep.int("Invalid URI scheme", len)
+        doi <- schemes[ind] == "doi"
+        if(any(doi))
+            msg[doi] <- paste(msg[doi], "(use \\doi for DOIs in Rd markup)")
         bad <- rbind(bad,
-                     .gather(urls[ind],
-                             parents[ind],
-                             m = rep.int("Invalid URI scheme", len)))
+                     .gather(urls[ind], parents[ind], m = msg))
     }
 
     ## ftp.
     pos <- which(schemes == "ftp")
-    if(length(pos)) {
+    if(length(pos) && remote) {
         results <- do.call(rbind, lapply(urls[pos], .check_ftp))
         status <- as.numeric(results[, 1L])
         ind <- (status < 0L) | (status >= 400L)
@@ -437,21 +490,23 @@ function(db, verbose = FALSE)
         status <- as.numeric(results[, 1L])
         ## 405 is HTTP not allowing HEAD requests
         ## maybe also skip 500, 503, 504 as likely to be temporary issues
-        ind <- !(status %in% c(200L, 405L)) |
-            nzchar(results[, 4L]) | nzchar(results[, 5L])
+        ind <- is.na(match(status, c(200L, 405L, NA))) |
+            nzchar(results[, 4L]) |
+            nzchar(results[, 5L]) |
+            nzchar(results[, 6L])
         if(any(ind)) {
             pos <- pos[ind]
             s <- as.character(status[ind])
+            s[is.na(s)] <- ""
             s[s == "-1"] <- "Error"
             m <- results[ind, 2L]
             m[is.na(m)] <- ""
-            url <- urls[pos]; newLoc <- results[ind, 3L]
-            ind2 <- nzchar(newLoc)
-            url[ind2] <-
-                paste0(url[ind2], " (moved to ", newLoc[ind2], ")")
             bad <- rbind(bad,
-                         .gather(url, parents[pos], s, m,
-                                 results[ind, 4L], results[ind, 5L]))
+                         .gather(urls[pos], parents[pos], s, m,
+                                 results[ind, 3L],
+                                 results[ind, 4L],
+                                 results[ind, 5L],
+                                 results[ind, 6L]))
         }
     }
     bad
@@ -462,7 +517,12 @@ function(x, ...)
 {
     if(!NROW(x)) return(character())
 
-    paste0(sprintf("URL: %s", x$URL),
+    u <- x$URL
+    new <- x$New
+    ind <- nzchar(new)
+    u[ind] <- sprintf("%s (moved to %s)", u[ind], new[ind])
+
+    paste0(sprintf("URL: %s", u),
            sprintf("\nFrom: %s",
                    sapply(x$From, paste, collapse = "\n      ")),
            ifelse((s <- x$Status) == "",
@@ -476,7 +536,10 @@ function(x, ...)
                   "\nURL contains spaces"),
            ifelse((m <- x$CRAN) == "",
                   "",
-                  "\nCRAN URL not in canonical form")
+                  "\nCRAN URL not in canonical form"),
+           ifelse((m <- x$R) == "",
+                  "",
+                  "\nR-project URL not in canonical form")
            )
 }
 
@@ -487,3 +550,15 @@ function(x, ...)
         writeLines(paste(format(x), collapse = "\n\n"))
     invisible(x)
 }
+
+.curl_GET_status <-
+function(u, verbose = FALSE)
+{
+    if(verbose)
+        message(sprintf("processing %s", u))
+    g <- tryCatch(curl::curl_fetch_memory(u), error = identity)
+    if(inherits(g, "error"))
+        -1L
+    else
+        g$status_code
+}
