@@ -138,7 +138,7 @@ pGEDevDesc GEcurrentDevice(void)
     if (NoDevices()) {
 	SEXP defdev = GetOption1(install("device"));
 	if (isString(defdev) && length(defdev) > 0) {
-	    SEXP devName = installChar(STRING_ELT(defdev, 0));
+	    SEXP devName = installTrChar(STRING_ELT(defdev, 0));
 	    /*  Not clear where this should be evaluated, since
 		grDevices need not be in the search path.
 		So we look for it first on the global search path.
