@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2015 The R Core Team
+ *  Copyright (C) 2015-2017 The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,16 +29,67 @@
 #include <Fileio.h>
 #include <errno.h>
 
+#ifdef HAVE_UNISTD_H
+// for unlink
+# include <unistd.h>
+#endif
+
 #ifdef HAVE_LIBCURL
 # include <curl/curl.h>
 /*
-  This needs libcurl >= 7.28.0 (Oct 2012) for curl_multi_wait.
+  This needed libcurl >= 7.28.0 (Oct 2012) for curl_multi_wait.
+  Substitute code is provided for a Unix-alike only.
+
   There is a configure test but it is not used on Windows and system
   software can change.
 */
-# if LIBCURL_VERSION_MAJOR < 7 || (LIBCURL_VERSION_MAJOR == 7 && LIBCURL_VERSION_MINOR < 28)
-# error libcurl 7.28.0 or later is required.
+# ifdef Win32
+#  if LIBCURL_VERSION_MAJOR < 7 || (LIBCURL_VERSION_MAJOR == 7 && LIBCURL_VERSION_MINOR < 28)
+#  error libcurl 7.28.0 or later is required.
+#  endif
+# else
+#  if LIBCURL_VERSION_MAJOR < 7 || (LIBCURL_VERSION_MAJOR == 7 && LIBCURL_VERSION_MINOR < 22)
+#  error libcurl 7.22.0 or later is required.
+#  endif
 # endif
+extern void Rsleep(double timeint);
+#endif
+
+# if (LIBCURL_VERSION_MAJOR == 7 && LIBCURL_VERSION_MINOR < 28)
+
+// curl/curl.h includes <sys/select.h> and headers it requires.
+
+#define curl_multi_wait R_curl_multi_wait
+
+static CURLMcode
+R_curl_multi_wait(CURLM *multi_handle,
+		  /* IGNORED */ void *unused,
+		  /* IGNORED */ unsigned int extra,
+		  int timeout_ms, int *ret)
+{
+    fd_set fdread;
+    fd_set fdwrite;
+    fd_set fdexcep;
+    FD_ZERO(&fdread);
+    FD_ZERO(&fdwrite);
+    FD_ZERO(&fdexcep);
+
+    struct timeval timeout;
+
+    timeout.tv_sec = timeout_ms / 1000;
+    timeout.tv_usec = (timeout_ms % 1000) * 1000;
+
+    int maxfd = -1;
+    CURLMcode
+	mc = curl_multi_fdset(multi_handle, &fdread, &fdwrite, &fdexcep, &maxfd);
+    if (maxfd == -1) {
+	*ret = 0;
+	Rsleep(0.1);
+    } else
+	*ret = select(maxfd+1, &fdread, &fdwrite, &fdexcep, &timeout);
+
+    return mc;
+}
 #endif
 
 SEXP attribute_hidden in_do_curlVersion(SEXP call, SEXP op, SEXP args, SEXP rho)
@@ -114,8 +165,8 @@ static const char *ftp_errstr(const long status)
     case 450: str = "Requested file action not taken"; break;
     case 451: str = "Requested action aborted; local error in processing"; break;
     case 452:
-        str = "Requested action not taken; insufficient storage space in system";
-        break;
+	str = "Requested action not taken; insufficient storage space in system";
+	break;
     case 501: str = "Syntax error in parameters or arguments"; break;
     case 502: str = "Command not implemented"; break;
     case 503: str = "Bad sequence of commands"; break;
@@ -123,12 +174,12 @@ static const char *ftp_errstr(const long status)
     case 530: str = "Not logged in"; break;
     case 532: str = "Need account for storing files"; break;
     case 550:
-        str = "Requested action not taken; file unavailable";
-        break;
+	str = "Requested action not taken; file unavailable";
+	break;
     case 551: str = "Requested action aborted; page type unknown"; break;
     case 552:
-        str = "Requested file action aborted; exceeded storage allocation";
-        break;
+	str = "Requested file action aborted; exceeded storage allocation";
+	break;
     case 553: str = "Requested action not taken; file name not allowed"; break;
     default: str = "Unknown Error"; break;
     }
@@ -146,23 +197,27 @@ static int curlMultiCheckerrs(CURLM *mhnd)
     for(int n = 1; n > 0;) {
 	CURLMsg *msg = curl_multi_info_read(mhnd, &n);
 	if (msg && (msg->data.result != CURLE_OK)) {
-	    const char *url, *strerr;
+	    const char *url, *strerr, *type;
 	    long status = 0;
 	    curl_easy_getinfo(msg->easy_handle, CURLINFO_EFFECTIVE_URL, &url);
 	    curl_easy_getinfo(msg->easy_handle, CURLINFO_RESPONSE_CODE,
 			      &status);
+	    // This reports the redirected URL
 	    if (status >= 400) {
-		if (url && url[0] == 'h')
+		if (url && url[0] == 'h') {
 		    strerr = http_errstr(status);
-		else
+		    type = "HTTP";
+		} else {
 		    strerr = ftp_errstr(status);
-		warning(_("URL '%s': status was '%d %s'"), url, status,
-			strerr);
+		    type = "FTP";
+		}
+		warning(_("cannot open URL '%s': %s status was '%d %s'"),
+			url, type, status, strerr);
 	    } else {
 		strerr = curl_easy_strerror(msg->data.result);
 		warning(_("URL '%s': status was '%s'"), url, strerr);
 	    }
-	    retval += 1;
+	    retval++;
 	}
     }
     return retval;
@@ -182,7 +237,7 @@ static void curlCommon(CURL *hnd, int redirect, int verify)
 	curl_easy_setopt(hnd, CURLOPT_SSL_VERIFYHOST, 0L);
 	curl_easy_setopt(hnd, CURLOPT_SSL_VERIFYPEER, 0L);
     }
-    // for consistency, but all that does is look up an option.
+    // for consistency, but all utils:::makeUserAgent does is look up an option.
     SEXP sMakeUserAgent = install("makeUserAgent");
     SEXP agentFun = PROTECT(lang2(sMakeUserAgent, ScalarLogical(0)));
     SEXP utilsNS = PROTECT(R_FindNamespace(mkString("utils")));
@@ -292,7 +347,7 @@ static void putdashes(int *pold, int new)
     *pold = new;
 }
 
-#ifdef Win32
+# ifdef Win32
 // ------- Windows progress bar -----------
 #include <ga.h>
 
@@ -359,19 +414,22 @@ int progress(void *clientp, double dltotal, double dlnow,
     return 0;
 }
 
-#else
+# else
 // ------- Unix-alike progress bar -----------
 
 static
 int progress(void *clientp, double dltotal, double dlnow,
 	     double ultotal, double ulnow)
 {
+    CURL *hnd = (CURL *) clientp;
+    long status;
+    curl_easy_getinfo(hnd, CURLINFO_RESPONSE_CODE, &status);
+
     // we only use downloads.  dltotal may be zero.
-    if (dltotal > 0.) {
+    if ((status < 300) && (dltotal > 0.)) {
 	if (total == 0.) {
 	    total = dltotal;
 	    char *type = NULL;
-	    CURL *hnd = (CURL *) clientp;
 	    curl_easy_getinfo(hnd, CURLINFO_CONTENT_TYPE, &type);
 	    REprintf("Content type '%s'", type ? type : "unknown");
 	    if (total > 1024.0*1024.0)
@@ -389,10 +447,8 @@ int progress(void *clientp, double dltotal, double dlnow,
     }
     return 0;
 }
-#endif
-
-extern void Rsleep(double timeint);
-#endif
+# endif // Win32
+#endif // HAVE_LIBCURL
 
 /* download(url, destfile, quiet, mode, headers, cacheOK) */
 
@@ -446,7 +502,6 @@ in_do_curlDownload(SEXP call, SEXP op, SEXP args, SEXP rho)
     FILE *out[nurls];
 
     for(int i = 0; i < nurls; i++) {
-	out[i] = NULL;
 	url = CHAR(STRING_ELT(scmd, i));
 	hnd[i] = curl_easy_init();
 	curl_easy_setopt(hnd[i], CURLOPT_URL, url);
@@ -454,38 +509,25 @@ in_do_curlDownload(SEXP call, SEXP op, SEXP args, SEXP rho)
 	/* Users will normally expect to follow redirections, although
 	   that is not the default in either curl or libcurl. */
 	curlCommon(hnd[i], 1, 1);
+#if (LIBCURL_VERSION_MINOR >= 25)
 	curl_easy_setopt(hnd[i], CURLOPT_TCP_KEEPALIVE, 1L);
+#endif
 	if (!cacheOK)
 	    curl_easy_setopt(hnd[i], CURLOPT_HTTPHEADER, slist1);
 
-	/* check that connection can be opened... */
-	curl_easy_setopt(hnd[i], CURLOPT_NOPROGRESS, 1L);
-	curl_easy_setopt(hnd[i], CURLOPT_NOBODY, 1L);
-	curl_easy_setopt(hnd[i], CURLOPT_HEADERFUNCTION, &rcvHeaders);
-	curl_easy_setopt(hnd[i], CURLOPT_WRITEHEADER, &headers);
-	/* libcurl (at least 7.40.0) does not respect CURLOPT_NOBODY
-	   for some ftp header info (Content-Length and Accept-ranges). */
-	curl_easy_setopt(hnd[i], CURLOPT_WRITEFUNCTION, &rcvBody);
-	CURLcode ret = curl_easy_perform(hnd[i]);
-	curl_multi_add_handle(mhnd, hnd[i]);
-	if (ret != CURLE_OK) {
-	    n_err += 1;
-	    /* warning signalled via curlMultiCheckerrs */
-	    continue;
-	}
-	/* ...and that destfile can be written */
+	/* check that destfile can be written */
 	file = translateChar(STRING_ELT(sfile, i));
 	out[i] = R_fopen(R_ExpandFileName(file), mode);
 	if (!out[i]) {
 	    n_err += 1;
 	    warning(_("URL %s: cannot open destfile '%s', reason '%s'"),
 		    url, file, strerror(errno));
-	    continue;
+	    if (nurls == 1) break; else continue;
+	} else {
+	    // This uses the internal CURLOPT_WRITEFUNCTION
+	    curl_easy_setopt(hnd[i], CURLOPT_WRITEDATA, out[i]);
+	    curl_multi_add_handle(mhnd, hnd[i]);
 	}
-	curl_easy_setopt(hnd[i], CURLOPT_WRITEFUNCTION, NULL); /* necessary? */
-	curl_easy_setopt(hnd[i], CURLOPT_WRITEDATA, out[i]);
-	curl_easy_setopt(hnd[i], CURLOPT_NOBODY, 0L);
-	curl_easy_setopt(hnd[i], CURLOPT_HEADER, 0L);
 
 	total = 0.;
 	if (!quiet && nurls <= 1) {
@@ -500,13 +542,13 @@ in_do_curlDownload(SEXP call, SEXP op, SEXP args, SEXP rho)
 					   rect(0, 0, 540, 100),
 					   Titlebar | Centered);
 		    setbackground(pbar.wprog, dialog_bg());
-		    pbar.l_url = newlabel(" ", rect(10, 15, 520, 25), 
+		    pbar.l_url = newlabel(" ", rect(10, 15, 520, 25),
 					  AlignCenter);
 		    pbar.pb = newprogressbar(rect(20, 50, 500, 20),
 					     0, 1024, 1024, 1);
 		    pbar.pc = 0;
 		}
-	    
+
 		settext(pbar.l_url, url);
 		setprogressbar(pbar.pb, 0);
 		settext(pbar.wprog, "Download progress");
@@ -520,7 +562,7 @@ in_do_curlDownload(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    // For libcurl >= 7.32.0 use CURLOPT_XFERINFOFUNCTION
 	    curl_easy_setopt(hnd[i], CURLOPT_PROGRESSFUNCTION, progress);
 	    curl_easy_setopt(hnd[i], CURLOPT_PROGRESSDATA, hnd[i]);
-	}
+	} else curl_easy_setopt(hnd[i], CURLOPT_NOPROGRESS, 1L);
 
 	/* This would allow the negotiation of compressed HTTP transfers,
 	   but it is not clear it is always a good idea.
@@ -530,14 +572,22 @@ in_do_curlDownload(SEXP call, SEXP op, SEXP args, SEXP rho)
 	if (!quiet) REprintf(_("trying URL '%s'\n"), url);
     }
 
+    if (n_err == nurls) {
+	// no dest files could be opened, so bail out
+	curl_multi_cleanup(mhnd);
+	return ScalarInteger(1);
+    }
+
     R_Busy(1);
     //  curl_multi_wait needs curl >= 7.28.0 .
     curl_multi_perform(mhnd, &still_running);
     do {
 	int numfds;
 	CURLMcode mc = curl_multi_wait(mhnd, NULL, 0, 100, &numfds);
-	if (mc != CURLM_OK)  // internal, do not translate
-	    error("curl_multi_wait() failed, code %d", mc);
+	if (mc != CURLM_OK)  { // internal, do not translate
+	    warning("curl_multi_wait() failed, code %d", mc);
+	    break;
+	}
 	if (!numfds) {
 	    /* 'numfds' being zero means either a timeout or no file
 	       descriptors to wait for. Try timeout on first
@@ -563,9 +613,11 @@ in_do_curlDownload(SEXP call, SEXP op, SEXP args, SEXP rho)
     if (R_Consolefile) fflush(R_Consolefile);
 #endif
     if (nurls == 1) {
+	long status;
+	curl_easy_getinfo(hnd[0], CURLINFO_RESPONSE_CODE, &status);
 	double cl, dl;
 	curl_easy_getinfo(hnd[0], CURLINFO_SIZE_DOWNLOAD, &dl);
-	if (!quiet) {
+	if (!quiet && status == 200) {
 	    if (dl > 1024*1024)
 		REprintf("downloaded %0.1f MB\n\n", (double)dl/1024/1024);
 	    else if (dl > 10240)
@@ -581,16 +633,34 @@ in_do_curlDownload(SEXP call, SEXP op, SEXP args, SEXP rho)
     n_err += curlMultiCheckerrs(mhnd);
 
     for (int i = 0; i < nurls; i++) {
-	if (out[i])
-            fclose(out[i]);
+	if (out[i]) {
+	    fclose(out[i]);
+	    double dl;
+	    curl_easy_getinfo(hnd[i], CURLINFO_SIZE_DOWNLOAD, &dl);
+	    long status;
+	    curl_easy_getinfo(hnd[i], CURLINFO_RESPONSE_CODE, &status);
+	    // should we do something about incomplete transfers?
+	    if (status != 200 && dl == 0. && strchr(mode, 'w'))
+		unlink(R_ExpandFileName(translateChar(STRING_ELT(sfile, i))));
+	}
 	curl_multi_remove_handle(mhnd, hnd[i]);
 	curl_easy_cleanup(hnd[i]);
     }
+    long status = 0L;
+    if(nurls == 1)
+	curl_easy_getinfo(hnd[0], CURLINFO_RESPONSE_CODE, &status);
     curl_multi_cleanup(mhnd);
     if (!cacheOK) curl_slist_free_all(slist1);
 
-    if (n_err != 0)
-	error(_("cannot download all files"));
+    if(nurls > 1) {
+	if (n_err == nurls) error(_("cannot download any files"));
+	else if (n_err) warning(_("some files were not downloaded"));
+    } else if(n_err) {
+	if (status != 200)
+	    error(_("cannot open URL '%s'"), CHAR(STRING_ELT(scmd, 0)));
+	else
+	    error(_("download from '%s' failed"), CHAR(STRING_ELT(scmd, 0)));
+    }
 
     return ScalarInteger(0);
 #endif
@@ -668,12 +738,14 @@ static int fetchData(RCurlconn ctxt)
 {
     int repeats = 0;
     CURLM *mhnd = ctxt->mh;
-    
+
     do {
 	int numfds;
 	CURLMcode mc = curl_multi_wait(mhnd, NULL, 0, 100, &numfds);
-	if (mc != CURLM_OK)
-	    error("curl_multi_wait() failed, code %d", mc);
+	if (mc != CURLM_OK) {
+	    warning("curl_multi_wait() failed, code %d", mc);
+	    break;
+	}
 	if (!numfds) {
 	    if (repeats++ > 0) Rsleep(0.1);
 	} else repeats = 0;
@@ -700,11 +772,11 @@ static void Curl_destroy(Rconnection con)
     RCurlconn ctxt;
 
     if (NULL == con)
-        return;
+	return;
     ctxt = (RCurlconn)(con->private);
 
     if (NULL == ctxt)
-        return;
+	return;
 
     free(ctxt->buf);
     free(ctxt);
@@ -744,7 +816,9 @@ static Rboolean Curl_open(Rconnection con)
     curl_easy_setopt(ctxt->hnd, CURLOPT_FAILONERROR, 1L);
     curlCommon(ctxt->hnd, 1, 1);
     curl_easy_setopt(ctxt->hnd, CURLOPT_NOPROGRESS, 1L);
+#if (LIBCURL_VERSION_MINOR >= 25)
     curl_easy_setopt(ctxt->hnd, CURLOPT_TCP_KEEPALIVE, 1L);
+#endif
 
     curl_easy_setopt(ctxt->hnd, CURLOPT_WRITEFUNCTION, rcvData);
     curl_easy_setopt(ctxt->hnd, CURLOPT_WRITEDATA, ctxt);
@@ -760,7 +834,7 @@ static Rboolean Curl_open(Rconnection con)
 	n_err += fetchData(ctxt);
     if (n_err != 0) {
 	Curl_close(con);
-	error(_("cannot open connection"), n_err);
+	error(_("cannot open the connection to '%s'"), url);
     }
 
     con->isopen = TRUE;
@@ -783,7 +857,7 @@ static int Curl_fgetc_internal(Rconnection con)
 
 
 // 'type' is unused.
-Rconnection 
+Rconnection
 in_newCurlUrl(const char *description, const char * const mode, int type)
 {
 #ifdef HAVE_LIBCURL
@@ -793,14 +867,14 @@ in_newCurlUrl(const char *description, const char * const mode, int type)
     if (!new->class) {
 	free(new);
 	error(_("allocation of url connection failed"));
-        /* for Solaris 12.5 */ new = NULL;
+	/* for Solaris 12.5 */ new = NULL;
     }
     strcpy(new->class, "url-libcurl");
     new->description = (char *) malloc(strlen(description) + 1);
     if (!new->description) {
 	free(new->class); free(new);
 	error(_("allocation of url connection failed"));
-        /* for Solaris 12.5 */ new = NULL;
+	/* for Solaris 12.5 */ new = NULL;
     }
     init_con(new, description, CE_NATIVE, mode);
     new->canwrite = FALSE;
@@ -814,7 +888,7 @@ in_newCurlUrl(const char *description, const char * const mode, int type)
     if (!new->private) {
 	free(new->description); free(new->class); free(new);
 	error(_("allocation of url connection failed"));
-        /* for Solaris 12.5 */ new = NULL;
+	/* for Solaris 12.5 */ new = NULL;
     }
     RCurlconn ctxt = (RCurlconn) new->private;
     ctxt->bufsize = 2 * CURL_MAX_WRITE_SIZE;
