@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2015  The R Core Team
+ *  Copyright (C) 1997--2016  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@
 #endif
 
 #if defined(HAVE_SYS_RESOURCE_H) && defined(HAVE_GETRUSAGE)
-/* on MacOS X it seems sys/resource.h needs sys/time.h first */
+/* on macOS it seems sys/resource.h needs sys/time.h first */
 # include <sys/time.h>
 # include <sys/resource.h>
 #endif
@@ -270,7 +270,7 @@ SEXP attribute_hidden do_system(SEXP call, SEXP op, SEXP args, SEXP rho)
 		  cmd, strerror(errno));
 #ifdef HAVE_GETLINE
         size_t read;
-        for(i = 0; (read = getline(&buf, &buf_len, fp)) != -1; i++) {
+        for(i = 0; (read = getline(&buf, &buf_len, fp)) != (size_t)-1; i++) {
 	    if (buf[read - 1] == '\n')
 #else
 	for (i = 0; fgets(buf, INTERN_BUFSIZE, fp); i++) {
@@ -463,12 +463,6 @@ void R_ProcessEvents(void)
 # endif
 #endif
 
-#ifdef linux
-# ifdef HAVE_FPU_CONTROL_H
-#  include <fpu_control.h>
-# endif
-#endif
-
 /* patch from Ei-ji Nakama for Intel compilers on ix86.
    From http://www.nakama.ne.jp/memo/ia32_linux/R-2.1.1.iccftzdaz.patch.txt.
    Since updated to include x86_64.
@@ -486,9 +480,6 @@ void fpu_setup(Rboolean start)
     fpsetmask(0);
 #endif
 
-#ifdef NEED___SETFPUCW
-    __setfpucw(_FPU_IEEE);
-#endif
 #if (defined(__i386) || defined(__x86_64)) && defined(__INTEL_COMPILER) && __INTEL_COMPILER > 800
     _MM_SET_FLUSH_ZERO_MODE(_MM_FLUSH_ZERO_OFF);
     _MM_SET_DENORMALS_ZERO_MODE(_MM_DENORMALS_ZERO_OFF);
@@ -497,9 +488,5 @@ void fpu_setup(Rboolean start)
 #ifdef __FreeBSD__
     fpsetmask(~0);
 #endif
-
-#ifdef NEED___SETFPUCW
-    __setfpucw(_FPU_DEFAULT);
-#endif
     }
 }
