@@ -13,7 +13,7 @@ if(interactive()) {
 } else if(FALSE) { ## MM @ testing *manually* only
     options(error = recover, Matrix.verbose = TRUE, warn = 1)
 } else {
-    options(Matrix.verbose = TRUE, warn = 1)
+    options(                 Matrix.verbose = TRUE, warn = 1)
 }
 
 
@@ -72,6 +72,16 @@ stopifnot(identical(mn["rc", "D"], mn[3,4]), mn[3,4] == 24,
 	  )
 showProc.time()
 
+## Bug found thanks to Timothy Mak, Feb 3, 2017:
+## sparseMatrix logical indexing with (partial) NA:
+a.m <- as.matrix(mn)
+assert.EQ(as.matrix(ms), a.m) # incl. dimnames
+iN4 <- c(NA, TRUE, FALSE, TRUE)
+assert.EQ(as.matrix(mn[,iN4]), a.m[,iN4]) # (incl. dimnames)
+##assert.EQ(as.matrix(ms[,iN4]), a.m[,iN4]) # ms[, <with_NA>]  fails still :
+try(ms[,iN4])
+try(ms[,iN4] <- 100) ## <- segfaulted in Matrix <= 1.2-8  (!)
+
 ## R-forge Matrix bug #2556: Subsetting a sparse matrix did remove  names(dimnames(.)) :
 m44 <- matrix(1:16, 4, 4, dimnames=list(row=c('a','b','c','d'), col=c('x','y','z','w')))
 ## Dense matrix: ------------------------------------------
@@ -641,8 +651,10 @@ D[i] <- v; assert.EQ.mat(D,m) # ddi -> dtT -> dgT
 s[i] <- v; assert.EQ.mat(s,m) # dtT -> dgT
 S[i] <- v; assert.EQ.mat(S,m); S # dtC -> dtT -> dgT -> dgC
 m.L <- asLogical(m)
-C[i] <- v; assert.EQ.mat(C,m.L); validObject(C)
-N[i] <- v; assert.EQ.mat(N,m.L); validObject(N)
+C[i] <- v # - with a warning: C is nMatrix, v not T/F
+assert.EQ.mat(C,m.L); validObject(C)
+N[i] <- v # - with a warning
+assert.EQ.mat(N,m.L); validObject(N)
 stopifnot(Q.C.identical(D,s, checkClass=FALSE))
 ## logical *vector* indexing
 eval(.iniDiag.example)
@@ -651,7 +663,7 @@ M[L] <- z; assert.EQ.mat(M,m)
 D[L] <- z; assert.EQ.mat(D,m)
 s[L] <- z; assert.EQ.mat(s,m)
 S[L] <- z; assert.EQ.mat(S,m) ; S
-C[L] <- z; assert.EQ.mat(C,m.L)
+C[L] <- z; assert.EQ.mat(C,m.L) # with a good warning
 N[L] <- z; assert.EQ.mat(N,m.L)
 
 
@@ -705,10 +717,13 @@ M[i] <- v; assert.EQ.mat(M,m) # dge
 D[i] <- v; assert.EQ.mat(D,m) # ddi -> dtT -> dgT
 s[i] <- v; assert.EQ.mat(s,m) # dtT -> dgT
 S[i] <- v; assert.EQ.mat(S,m); S # dtC -> dtT -> dgT -> dgC
-N[i] <- v; assert.EQ.mat(N,m.L); N #
-C[i] <- v; assert.EQ.mat(C,m.L); C #
-
+N[i] <- v # with a good warning
+assert.EQ.mat(N,m.L); N
+C[i] <- v #  ..  .  ..  warning
+assert.EQ.mat(C,m.L); C #
 
+options(warn = 2) #---------------------# NO WARNINGS from here -----------------
+					# =====================
 ## 2) negative [i,j] indices
 mc <- mC[1:5, 1:7]
 mt <- mT[1:5, 1:7]
@@ -1039,4 +1054,75 @@ for(nm in ls()) if(is(.m <- get(nm), "Matrix")) {
 showProc.time()
 }#--------------end if(doExtras) -----------------------------------------------
 
+## Bugs found by Peter Ralph
+n <- 17
+x <- Matrix(0, n,n)
+## x must have at least three nonzero entries
+x[1,1] <- x[2,1:2] <- 1.
+x0 <- x <- as(x,"dgTMatrix")  # if x is dgCMatrix, no error
+##
+z <- matrix(x) # <== not the "Matrix way":  a (n, 1) matrix
+z[1] <- 0
+
+x[1:n, 1:n] <- as(z, "sparseVector")
+## gave Error: ... invalid subscript type 'S4'
+x2 <- x
+
+dim(zC <- as(z, "dgCMatrix"))
+x <- x0
+x[] <- zC # did fail, then gave warning.
+x1 <- x
+##
+x <- x0
+x[] <- as(zC, "sparseVector") # did fail, too
+x2 <- x
+stopifnot(identical(x1,x2))
+x <- as(x0, "matrix")
+x[] <- z
+assert.EQ.mat(x1, x)
+
+i <- 4:7
+x1 <- x0; x1[cbind(i, i+10)] <- i^2
+x2 <- x0; x2[cbind(i, i+10)] <- as.matrix(i^2)
+## failed: nargs() = 4 ... please report
+
+stopifnot(isValid(x1, "dgTMatrix"), identical(x1, x2))
+
+
+iv <- c(rep(0,100), 3, 0,0,7,0,0,0)
+sv0  <- sv  <- as(iv, "sparseVector")
+sv.0 <- sv. <- as(as.integer(iv), "sparseVector")
+stopifnot(canCoerce("integer", "sparseVector"))
+sv2 <- as(sv, "isparseVector")
+stopifnot(validObject(sv), validObject(sv2), identical(sv., sv2),
+          sv == sv.)
+n0 <- sv. != 0
+## --> ../R/sparseVector.R : replSPvec()
+if(interactive())  debug(Matrix:::replSPvec)
+##
+sv [n0] <- sv [n0]
+sv.[n0] <- sv.[n0] # gave error
+stopifnot(identical(sv , sv0),
+          identical(sv., sv.0))
+sv [3:7] <- 0
+sv.[3:7] <- 0L
+stopifnot(identical(sv , sv0), identical(sv., sv.0))
+sv [2:4] <- 2:4
+sv.[2:4] <- 2:4
+stopifnot(which(sv != 0) == (which(sv. != 0) -> in0),
+          in0 == c(2:4, 101L, 104L))
+sv [2:6] <- 0L
+sv.[2:6] <- 0L
+stopifnot(identical(sv , sv0), identical(sv., sv.0))
+
+## the next six *all* gave an error -- but should be no-op's:
+for(vv in list(sv, sv.0))
+    for(ind in list(0, FALSE, logical(length(vv))))
+        vv[ind] <- NA
+stopifnot(identical(sv , sv0), identical(sv., sv.0))
+
+
+
+showProc.time()
+
 if(!interactive()) warnings()
